@@ -114,13 +114,18 @@ struct _NemoAction {
     gchar *orig_label;
     gchar *orig_tt;
     gboolean use_parent_dir;
-    gboolean log_output;
     GList *dbus;
+    guint dbus_recalc_timeout_id;
+    GList *gsettings;
+    guint gsettings_recalc_timeout_id;
     gboolean dbus_satisfied;
+    gboolean gsettings_satisfied;
     gboolean escape_underscores;
     gboolean escape_space;
     gboolean show_in_blank_desktop;
     gboolean run_in_terminal;
+
+    gboolean constructing;
 };
 
 struct _NemoActionClass {
@@ -130,24 +135,11 @@ struct _NemoActionClass {
 GType         nemo_action_get_type             (void);
 NemoAction   *nemo_action_new                  (const gchar *name, const gchar *path);
 void          nemo_action_activate             (NemoAction *action, GList *selection, NemoFile *parent);
-SelectionType nemo_action_get_selection_type   (NemoAction *action);
-gchar       **nemo_action_get_extension_list   (NemoAction *action);
-gchar       **nemo_action_get_mimetypes_list   (NemoAction *action);
-void          nemo_action_set_key_file_path    (NemoAction *action, const gchar *path);
-void          nemo_action_set_exec             (NemoAction *action, const gchar *exec);
-void          nemo_action_set_parent_dir       (NemoAction *action, const gchar *parent_dir);
-void          nemo_action_set_separator        (NemoAction *action, const gchar *separator);
-void          nemo_action_set_conditions       (NemoAction *action, gchar **conditions);
-void          nemo_action_set_orig_label       (NemoAction *action, const gchar *orig_label);
-void          nemo_action_set_orig_tt          (NemoAction *action, const gchar *orig_tt);
+
 const gchar  *nemo_action_get_orig_label       (NemoAction *action);
 const gchar  *nemo_action_get_orig_tt          (NemoAction *action);
-gchar       **nemo_action_get_conditions       (NemoAction *action);
 gchar        *nemo_action_get_label            (NemoAction *action, GList *selection, NemoFile *parent);
 gchar        *nemo_action_get_tt               (NemoAction *action, GList *selection, NemoFile *parent);
-void          nemo_action_set_extensions       (NemoAction *action, gchar **extensions);
-void          nemo_action_set_mimetypes        (NemoAction *action, gchar **mimetypes);
-gboolean      nemo_action_get_dbus_satisfied   (NemoAction *action);
-gboolean      nemo_action_get_visibility       (NemoAction *action, GList *selection, NemoFile *parent);
+gboolean      nemo_action_get_visibility       (NemoAction *action, GList *selection, NemoFile *parent, gboolean for_places);
 
 #endif /* NEMO_ACTION_H */
