@@ -27,6 +27,7 @@
 #include <config.h>
 
 #include "nemo-bookmark.h"
+#include "nemo-metadata.h"
 
 #include <eel/eel-vfs-extensions.h>
 #include <gio/gio.h>
@@ -43,6 +44,7 @@
 enum {
 	CONTENTS_CHANGED,
     LOCATION_MOUNTED,
+    LOOKUP_METADATA,
 	LAST_SIGNAL
 };
 
@@ -50,7 +52,8 @@ enum {
 	PROP_NAME = 1,
 	PROP_CUSTOM_NAME,
 	PROP_LOCATION,
-	PROP_ICON,
+	PROP_ICON_NAME,
+    PROP_METADATA,
 	NUM_PROPERTIES
 };
 
@@ -64,10 +67,12 @@ struct NemoBookmarkDetails
 	char *name;
 	gboolean has_custom_name;
 	GFile *location;
-	GIcon *icon;
+    gchar *icon_name;
 	NemoFile *file;
 
 	char *scroll_file;
+
+    NemoBookmarkMetadata *metadata;
 };
 
 static void	  nemo_bookmark_disconnect_file	  (NemoBookmark	 *file);
@@ -89,29 +94,24 @@ nemo_bookmark_set_name_internal (NemoBookmark *bookmark,
 static void
 nemo_bookmark_update_icon (NemoBookmark *bookmark)
 {
-	GIcon *new_icon;
+    gchar *new_icon_name;
 
-	if (bookmark->details->file == NULL) {
-		return;
-	}
+    if (bookmark->details->file == NULL) {
+        return;
+    }
 
-	if (!nemo_file_is_local (bookmark->details->file)) {
-		/* never update icons for remote bookmarks */
-		return;
-	}
+    if (!nemo_file_is_not_yet_confirmed (bookmark->details->file) &&
+        nemo_file_check_if_ready (bookmark->details->file,
+                                  NEMO_FILE_ATTRIBUTES_FOR_ICON)) {
+        DEBUG ("%s: set new icon", nemo_bookmark_get_name (bookmark));
 
-	if (!nemo_file_is_not_yet_confirmed (bookmark->details->file) &&
-	    nemo_file_check_if_ready (bookmark->details->file,
-					  NEMO_FILE_ATTRIBUTES_FOR_ICON)) {
-		DEBUG ("%s: set new icon", nemo_bookmark_get_name (bookmark));
+        new_icon_name = nemo_file_get_control_icon_name (bookmark->details->file);
+        g_object_set (bookmark,
+                      "icon-name", new_icon_name,
+                      NULL);
 
-		new_icon = nemo_file_get_emblemed_icon (bookmark->details->file, 0);
-		g_object_set (bookmark,
-			      "icon", new_icon,
-			      NULL);
-
-		g_object_unref (new_icon);
-	}
+        g_free (new_icon_name);
+    }
 }
 
 static void
@@ -136,47 +136,103 @@ bookmark_set_name_from_ready_file (NemoBookmark *self,
 	g_free (display_name);
 }
 
-static void
-nemo_bookmark_set_icon_to_default (NemoBookmark *bookmark)
+static gchar *
+get_default_folder_icon_name (NemoBookmark *bookmark)
 {
-    GIcon *icon, *emblemed_icon, *folder;
-    GEmblem *emblem;
-    char *uri;
+    gchar *ret = NULL;
 
     if (g_file_is_native (bookmark->details->location)) {
-        folder = g_themed_icon_new (NEMO_ICON_FOLDER);
+        ret = g_strdup (NEMO_ICON_SYMBOLIC_FOLDER);
     } else {
-        uri = g_file_get_uri (bookmark->details->location);
+        gchar *uri = g_file_get_uri (bookmark->details->location);
         if (g_str_has_prefix (uri, EEL_SEARCH_URI)) {
-            folder = g_themed_icon_new (NEMO_ICON_FOLDER_SAVED_SEARCH);
+            ret = g_strdup (NEMO_ICON_SYMBOLIC_FOLDER_SAVED_SEARCH);
         } else {
-            folder = g_themed_icon_new (NEMO_ICON_FOLDER_REMOTE);
+            ret = g_strdup (NEMO_ICON_SYMBOLIC_FOLDER_REMOTE);
         }
         g_free (uri);
     }
 
+    return ret;
+}
+
+static gchar *
+construct_default_icon_from_metadata (NemoBookmark *bookmark)
+{
+    return get_default_folder_icon_name (bookmark);
+
+/* TODO: Remove emblem stuff from bookmarks */
+    // if (ret != NULL && md->emblems != NULL) {
+    //     guint i = 0;
+
+    //     GIcon *emb_icon;
+    //     GEmblem *emblem;
+
+    //     emb_icon = g_themed_icon_new (md->emblems[i]);
+    //     emblem = g_emblem_new (emb_icon);
+
+    //     ret = g_emblemed_icon_new (ret, emblem);
+
+    //     i++;
+
+    //     while (i < g_strv_length (md->emblems)) {
+    //         emb_icon = g_themed_icon_new (md->emblems[i]);
+    //         emblem = g_emblem_new (emb_icon);
+
+    //         g_emblemed_icon_add_emblem (G_EMBLEMED_ICON (ret), emblem);
+
+    //         i++;
+    //     }
+    // }
+
+    // return ret;
+}
+
+static void
+nemo_bookmark_set_icon_to_default (NemoBookmark *bookmark)
+{
+    gchar *icon_name;
+
+    icon_name = construct_default_icon_from_metadata (bookmark);
+
     if (!nemo_bookmark_uri_get_exists (bookmark)) {
-        DEBUG ("%s: file does not exist, add emblem", nemo_bookmark_get_name (bookmark));
+        DEBUG ("%s: file does not exist, use special icon", nemo_bookmark_get_name (bookmark));
 
-        icon = g_themed_icon_new (GTK_STOCK_DIALOG_WARNING);
-        emblem = g_emblem_new (icon);
+        g_clear_pointer (&icon_name, g_free);
 
-        emblemed_icon = g_emblemed_icon_new (folder, emblem);
-
-        g_object_unref (emblem);
-        g_object_unref (icon);
-        g_object_unref (folder);
-
-        folder = emblemed_icon;
+        icon_name = g_strdup (NEMO_ICON_SYMBOLIC_MISSING_BOOKMARK);
     }
 
     DEBUG ("%s: setting icon to default", nemo_bookmark_get_name (bookmark));
 
     g_object_set (bookmark,
-              "icon", folder,
-              NULL);
+                  "icon-name", icon_name,
+                  NULL);
 
-    g_object_unref (folder);
+    g_free (icon_name);
+}
+
+static gboolean
+metadata_changed (NemoBookmark *bookmark)
+{
+    gboolean ret = FALSE;
+    NemoBookmarkMetadata *data = nemo_bookmark_get_updated_metadata (bookmark);
+    gboolean has_custom = data && data->emblems;
+
+    gboolean had_custom = bookmark->details->metadata != NULL;
+
+    if ((has_custom && !had_custom) ||
+       (had_custom && !has_custom)) {
+        ret = TRUE;
+
+    } else if (has_custom && had_custom) {
+        NemoBookmarkMetadata *md = bookmark->details->metadata;
+        ret = nemo_bookmark_metadata_compare (data, md);
+    }
+
+    nemo_bookmark_metadata_free (data);
+
+    return ret;
 }
 
 static void
@@ -207,9 +263,9 @@ bookmark_file_changed_callback (NemoFile *file,
 	if (nemo_file_is_gone (file) ||
 	    nemo_file_is_in_trash (file)) {
 		/* The file we were monitoring has been trashed, deleted,
-		 * or moved in a way that we didn't notice. We should make 
-		 * a spanking new NemoFile object for this 
-		 * location so if a new file appears in this place 
+		 * or moved in a way that we didn't notice. We should make
+		 * a spanking new NemoFile object for this
+		 * location so if a new file appears in this place
 		 * we will notice. However, we can't immediately do so
 		 * because creating a new NemoFile directly as a result
 		 * of noticing a file goes away may trigger i/o on that file
@@ -225,6 +281,10 @@ bookmark_file_changed_callback (NemoFile *file,
 	} else {
 		nemo_bookmark_update_icon (bookmark);
 		bookmark_set_name_from_ready_file (bookmark, file);
+
+        if (metadata_changed (bookmark)) {
+            g_signal_emit (bookmark, signals[CONTENTS_CHANGED], 0);
+        }
 	}
 }
 
@@ -267,7 +327,7 @@ nemo_bookmark_connect_file (NemoBookmark *bookmark)
 	/* Set icon based on available information. */
 	nemo_bookmark_update_icon (bookmark);
 
-	if (bookmark->details->icon == NULL) {
+	if (bookmark->details->icon_name == NULL) {
 		nemo_bookmark_set_icon_to_default (bookmark);
 	}
 
@@ -290,17 +350,16 @@ nemo_bookmark_set_property (GObject *object,
 				GParamSpec *pspec)
 {
 	NemoBookmark *self = NEMO_BOOKMARK (object);
-	GIcon *new_icon;
-
 	switch (property_id) {
-	case PROP_ICON:
-		new_icon = g_value_get_object (value);
-
-		if (new_icon != NULL && !g_icon_equal (self->details->icon, new_icon)) {
-			g_clear_object (&self->details->icon);
-			self->details->icon = g_object_ref (new_icon);
-		}
+	case PROP_ICON_NAME:
+        ;
+        const gchar *new_icon_name;
 
+        new_icon_name = g_value_get_string (value);
+        if (new_icon_name != NULL && g_strcmp0 (self->details->icon_name, new_icon_name) != 0) {
+            g_clear_pointer (&self->details->icon_name, g_free);
+            self->details->icon_name = g_strdup (new_icon_name);
+        }
 		break;
 	case PROP_LOCATION:
 		self->details->location = g_value_dup_object (value);
@@ -311,6 +370,12 @@ nemo_bookmark_set_property (GObject *object,
 	case PROP_NAME:
 		nemo_bookmark_set_name_internal (self, g_value_get_string (value));
 		break;
+    case PROP_METADATA:
+        if (self->details->metadata)
+            g_clear_pointer (&self->details->metadata, nemo_bookmark_metadata_free);
+
+        self->details->metadata = g_value_get_pointer (value);
+        break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
 		break;
@@ -329,8 +394,8 @@ nemo_bookmark_get_property (GObject *object,
 	case PROP_NAME:
 		g_value_set_string (value, self->details->name);
 		break;
-	case PROP_ICON:
-		g_value_set_object (value, self->details->icon);
+	case PROP_ICON_NAME:
+		g_value_set_string (value, self->details->icon_name);
 		break;
 	case PROP_LOCATION:
 		g_value_set_object (value, self->details->location);
@@ -338,6 +403,9 @@ nemo_bookmark_get_property (GObject *object,
 	case PROP_CUSTOM_NAME:
 		g_value_set_boolean (value, self->details->has_custom_name);
 		break;
+    case PROP_METADATA:
+        g_value_set_pointer (value, self->details->metadata);
+        break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
 		break;
@@ -353,10 +421,12 @@ nemo_bookmark_finalize (GObject *object)
 
 	bookmark = NEMO_BOOKMARK (object);
 
-	nemo_bookmark_disconnect_file (bookmark);	
+	nemo_bookmark_disconnect_file (bookmark);
 
 	g_object_unref (bookmark->details->location);
-	g_clear_object (&bookmark->details->icon);
+	g_clear_pointer (&bookmark->details->icon_name, g_free);
+
+    g_clear_pointer (&bookmark->details->metadata, nemo_bookmark_metadata_free);
 
 	g_free (bookmark->details->name);
 	g_free (bookmark->details->scroll_file);
@@ -413,13 +483,19 @@ nemo_bookmark_class_init (NemoBookmarkClass *class)
 				     G_TYPE_FILE,
 				     G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_CONSTRUCT_ONLY);
 
-	properties[PROP_ICON] =
-		g_param_spec_object ("icon",
-				     "Bookmark's icon",
-				     "The icon of this bookmark",
-				     G_TYPE_ICON,
+	properties[PROP_ICON_NAME] =
+		g_param_spec_string ("icon-name",
+				     "Bookmark's icon name",
+				     "The icon name for this bookmark",
+				     NULL,
 				     G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
+    properties[PROP_METADATA] =
+        g_param_spec_pointer ("metadata",
+                     "Bookmark's non-gvfs metadata",
+                     "Metadata for defining the bookmark's icon",
+                     G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
 	g_object_class_install_properties (oclass, NUM_PROPERTIES, properties);
 
 	g_type_class_add_private (class, sizeof (NemoBookmarkDetails));
@@ -476,11 +552,11 @@ nemo_bookmark_set_custom_name (NemoBookmark *bookmark,
  * Check whether two bookmarks are considered identical.
  * @a: first NemoBookmark*.
  * @b: second NemoBookmark*.
- * 
- * Return value: 0 if @a and @b have same name and uri, 1 otherwise 
+ *
+ * Return value: 0 if @a and @b have same name and uri, 1 otherwise
  * (GCompareFunc style)
  **/
-int		    
+int
 nemo_bookmark_compare_with (gconstpointer a, gconstpointer b)
 {
 	NemoBookmark *bookmark_a;
@@ -496,12 +572,12 @@ nemo_bookmark_compare_with (gconstpointer a, gconstpointer b)
 			   bookmark_b->details->location)) {
 		return 1;
 	}
-	
+
 	if (g_strcmp0 (bookmark_a->details->name,
 		       bookmark_b->details->name) != 0) {
 		return 1;
 	}
-	
+
 	return 0;
 }
 
@@ -511,11 +587,11 @@ nemo_bookmark_compare_with (gconstpointer a, gconstpointer b)
  * Check whether the uris of two bookmarks are for the same location.
  * @a: first NemoBookmark*.
  * @b: second NemoBookmark*.
- * 
- * Return value: 0 if @a and @b have matching uri, 1 otherwise 
+ *
+ * Return value: 0 if @a and @b have matching uri, 1 otherwise
  * (GCompareFunc style)
  **/
-int		    
+int
 nemo_bookmark_compare_uris (gconstpointer a, gconstpointer b)
 {
 	NemoBookmark *bookmark_a;
@@ -536,23 +612,24 @@ nemo_bookmark_copy (NemoBookmark *bookmark)
 {
 	g_return_val_if_fail (NEMO_IS_BOOKMARK (bookmark), NULL);
 
-	return nemo_bookmark_new (
-			bookmark->details->location,
-			bookmark->details->has_custom_name ?
-			bookmark->details->name : NULL,
-			bookmark->details->icon);
+    return nemo_bookmark_new (bookmark->details->location,
+                              bookmark->details->has_custom_name ?
+                                  bookmark->details->name : NULL,
+                              bookmark->details->icon_name,
+                              bookmark->details->metadata ?
+                                  nemo_bookmark_metadata_copy (bookmark->details->metadata) : NULL);
 }
 
-GIcon *
-nemo_bookmark_get_icon (NemoBookmark *bookmark)
+gchar *
+nemo_bookmark_get_icon_name (NemoBookmark *bookmark)
 {
 	g_return_val_if_fail (NEMO_IS_BOOKMARK (bookmark), NULL);
 
 	/* Try to connect a file in case file exists now but didn't earlier. */
 	nemo_bookmark_connect_file (bookmark);
 
-	if (bookmark->details->icon) {
-		return g_object_ref (bookmark->details->icon);
+	if (bookmark->details->icon_name) {
+		return g_strdup (bookmark->details->icon_name);
 	}
 	return NULL;
 }
@@ -565,8 +642,8 @@ nemo_bookmark_get_location (NemoBookmark *bookmark)
 	/* Try to connect a file in case file exists now but didn't earlier.
 	 * This allows a bookmark to update its image properly in the case
 	 * where a new file appears with the same URI as a previously-deleted
-	 * file. Calling connect_file here means that attempts to activate the 
-	 * bookmark will update its image if possible. 
+	 * file. Calling connect_file here means that attempts to activate the
+	 * bookmark will update its image if possible.
 	 */
 	nemo_bookmark_connect_file (bookmark);
 
@@ -586,9 +663,10 @@ nemo_bookmark_get_uri (NemoBookmark *bookmark)
 }
 
 NemoBookmark *
-nemo_bookmark_new (GFile *location,
-		       const gchar *custom_name,
-                       GIcon *icon)
+nemo_bookmark_new (GFile                *location,
+                   const gchar          *custom_name,
+                   const gchar          *icon_name,
+                   NemoBookmarkMetadata *md)
 {
 	NemoBookmark *new_bookmark;
     gchar *name;
@@ -598,37 +676,40 @@ nemo_bookmark_new (GFile *location,
     else
         name = g_strdup (custom_name);
 
-	new_bookmark = NEMO_BOOKMARK (g_object_new (NEMO_TYPE_BOOKMARK,
-							"location", location,
-							"icon", icon,
-							"name", name,
-							"custom-name", custom_name != NULL,
-							NULL));
+    new_bookmark = NEMO_BOOKMARK (g_object_new (NEMO_TYPE_BOOKMARK,
+						"location", location,
+						"icon-name", icon_name,
+						"name", name,
+						"custom-name", custom_name != NULL,
+						"metadata", md,
+						NULL));
     g_free (name);
 
-	return new_bookmark;
-}				 
+    return new_bookmark;
+}
 
 static GtkWidget *
 create_image_widget_for_bookmark (NemoBookmark *bookmark)
 {
-	GIcon *icon;
-	GtkWidget *widget;
+    GtkWidget *widget;
+    gchar *icon_name;
 
-	icon = nemo_bookmark_get_icon (bookmark);
-        widget = gtk_image_new_from_gicon (icon, GTK_ICON_SIZE_MENU);
-	g_object_unref (icon);
+    icon_name = nemo_bookmark_get_icon_name (bookmark);
+
+    widget = gtk_image_new_from_icon_name (icon_name, GTK_ICON_SIZE_MENU);
+
+    g_free (icon_name);
 
 	return widget;
 }
 
 /**
  * nemo_bookmark_menu_item_new:
- * 
+ *
  * Return a menu item representing a bookmark.
  * @bookmark: The bookmark the menu item represents.
  * Return value: A newly-created bookmark, not yet shown.
- **/ 
+ **/
 GtkWidget *
 nemo_bookmark_menu_item_new (NemoBookmark *bookmark)
 {
@@ -691,4 +772,106 @@ void
 nemo_bookmark_connect (NemoBookmark *bookmark)
 {
     nemo_bookmark_connect_file (bookmark);
-}
\ No newline at end of file
+}
+
+static gchar **
+char_list_to_strv (GList *list)
+{
+    GList *iter;
+
+    GPtrArray *array = g_ptr_array_new ();
+
+    for (iter = list; iter != NULL; iter = iter->next) {
+        g_ptr_array_add (array, g_strdup (iter->data));
+    }
+
+    g_ptr_array_add (array, NULL);
+
+    return (char **) g_ptr_array_free (array, FALSE);
+}
+
+NemoBookmarkMetadata *
+nemo_bookmark_get_updated_metadata (NemoBookmark  *bookmark)
+{
+    NemoBookmarkMetadata *ret = NULL;
+
+    if (!bookmark->details->file)
+        return NULL;
+
+    if (bookmark->details->file && !nemo_file_is_gone (bookmark->details->file)) {
+        GList *custom_emblems = NULL;
+
+        custom_emblems = nemo_file_get_metadata_list (bookmark->details->file, NEMO_METADATA_KEY_EMBLEMS);
+
+        if (custom_emblems) {
+            ret = nemo_bookmark_metadata_new ();
+            ret->emblems = char_list_to_strv (custom_emblems);
+
+            g_list_free_full (custom_emblems, g_free);
+        }
+
+    } else if (bookmark->details->metadata) {
+        ret = nemo_bookmark_metadata_copy (bookmark->details->metadata);
+    }
+
+    return ret;
+}
+
+NemoBookmarkMetadata *
+nemo_bookmark_get_current_metadata (NemoBookmark *bookmark)
+{
+    if (bookmark->details->metadata)
+        return bookmark->details->metadata;
+
+    return NULL;
+}
+
+NemoBookmarkMetadata *
+nemo_bookmark_metadata_new (void)
+{
+    NemoBookmarkMetadata *meta = g_slice_new0 (NemoBookmarkMetadata);
+
+    return meta;
+}
+
+NemoBookmarkMetadata *
+nemo_bookmark_metadata_copy (NemoBookmarkMetadata *meta)
+{
+    NemoBookmarkMetadata *copy = nemo_bookmark_metadata_new ();
+
+    copy->bookmark_name = g_strdup (meta->bookmark_name);
+    copy->emblems = g_strdupv (meta->emblems);
+
+    return copy;
+}
+
+gboolean
+nemo_bookmark_metadata_compare (NemoBookmarkMetadata *d1,
+                                NemoBookmarkMetadata *d2)
+{
+    if (g_strcmp0 (d1->bookmark_name, d2->bookmark_name) != 0 ||
+        (g_strv_length (d1->emblems) != g_strv_length (d2->emblems)))
+        return FALSE;
+
+    guint i;
+
+    for (i = 0; i < g_strv_length (d1->emblems); i++) {
+        if (g_strcmp0 (d1->emblems[i], d2->emblems[i]) != 0)
+            return FALSE;
+    }
+
+    return TRUE;
+}
+
+void
+nemo_bookmark_metadata_free (NemoBookmarkMetadata *metadata)
+{
+    if (metadata == NULL) {
+        return;
+    }
+
+    g_free (metadata->bookmark_name);
+    g_strfreev (metadata->emblems);
+
+    g_slice_free (NemoBookmarkMetadata, metadata);
+}
