@@ -154,7 +154,6 @@ nemo_directory_finalize (GObject *object)
 
 	nemo_directory_cancel (directory);
 	g_assert (directory->details->count_in_progress == NULL);
-	g_assert (directory->details->top_left_read_state == NULL);
 
 	if (directory->details->monitor_list != NULL) {
 		g_warning ("destroying a NemoDirectory while it's being monitored");
@@ -304,10 +303,6 @@ add_preferences_callbacks (void)
 				  "changed::" NEMO_PREFERENCES_SHOW_HIDDEN_FILES,
 				  G_CALLBACK(filtering_changed_callback),
 				  NULL);
-	g_signal_connect_swapped (nemo_preferences,
-				  "changed::" NEMO_PREFERENCES_SHOW_TEXT_IN_ICONS,
-				  G_CALLBACK (async_data_preference_changed_callback),
-				  NULL);
 	g_signal_connect_swapped (nemo_preferences,
 				  "changed::" NEMO_PREFERENCES_SHOW_DIRECTORY_ITEM_COUNTS,
 				  G_CALLBACK (async_data_preference_changed_callback),
@@ -554,6 +549,19 @@ nemo_directory_is_in_recent (NemoDirectory *directory)
    return g_file_has_uri_scheme (directory->details->location, "recent");
 }
 
+gboolean
+nemo_directory_is_in_admin (NemoDirectory *directory)
+{
+    g_assert (NEMO_IS_DIRECTORY (directory));
+
+    if (directory->details->location == NULL)
+    {
+        return FALSE;
+    }
+
+    return g_file_has_uri_scheme (directory->details->location, "admin");
+}
+
 gboolean
 nemo_directory_are_all_files_seen (NemoDirectory *directory)
 {
@@ -1022,6 +1030,8 @@ nemo_directory_notify_files_changed (GList *files)
 	GHashTable *changed_lists;
 	GList *node;
 	GFile *location;
+    GFile *parent;
+    NemoDirectory *dir;
 	NemoFile *file;
 
 	/* Make a list of changed files in each directory. */
@@ -1038,15 +1048,30 @@ nemo_directory_notify_files_changed (GList *files)
 			 * a changed signal.
 			 */
 			file->details->file_info_is_up_to_date = FALSE;
-			file->details->top_left_text_is_up_to_date = FALSE;
 			file->details->link_info_is_up_to_date = FALSE;
 			nemo_file_invalidate_extension_info_internal (file);
 
 			hash_table_list_prepend (changed_lists,
 						 file->details->directory,
 						 file);
-		}
-	}
+        } else {
+            parent = g_file_get_parent (location);
+            dir = nemo_directory_get_existing (parent);
+            if (dir != NULL && dir->details->new_files_in_progress != NULL &&
+                files != dir->details->new_files_in_progress_changes) {
+                dir->details->new_files_in_progress_changes = g_list_prepend (dir->details->new_files_in_progress_changes,
+                                                                              g_object_ref (location));
+            }
+
+            if (dir != NULL) {
+                nemo_directory_unref (dir);
+            }
+
+            if (parent != NULL) {
+                g_object_unref (parent);
+            }
+        }
+    }
 
 	/* Now send out the changed signals. */
 	g_hash_table_foreach (changed_lists, call_files_changed_unref_free_list, NULL);
@@ -1403,7 +1428,6 @@ nemo_directory_schedule_position_set (GList *position_setting_list)
 	GList *p;
 	const NemoFileChangesQueuePosition *item;
 	NemoFile *file;
-	char str[64];
 	time_t now;
 
 	time (&now);
@@ -1412,48 +1436,20 @@ nemo_directory_schedule_position_set (GList *position_setting_list)
 		item = (NemoFileChangesQueuePosition *) p->data;
 
 		file = nemo_file_get (item->location);
-		
-		if (item->set) {
-			g_snprintf (str, sizeof (str), "%d,%d", item->point.x, item->point.y);
-		} else {
-			str[0] = 0;
-		}
-		nemo_file_set_metadata
-			(file,
-			 NEMO_METADATA_KEY_ICON_POSITION,
-			 NULL,
-			 str);
 
 		if (item->set) {
-			nemo_file_set_time_metadata
-				(file,
-				 NEMO_METADATA_KEY_ICON_POSITION_TIMESTAMP,
-				 now);
+            nemo_file_set_position (file, item->point.x, item->point.y);
 		} else {
-			nemo_file_set_time_metadata
-				(file,
-				 NEMO_METADATA_KEY_ICON_POSITION_TIMESTAMP,
-				 UNDEFINED_TIME);
+            nemo_file_set_position (file, -1, -1);
 		}
 
-		if (item->set) {
-			g_snprintf (str, sizeof (str), "%d", item->screen);
-		} else {
-			str[0] = 0;
-		}
-		nemo_file_set_metadata
-			(file,
-			 NEMO_METADATA_KEY_SCREEN,
-			 NULL,
-			 str);
-		
         if (item->set) {
-            g_snprintf (str, sizeof (str), "%d", item->monitor);
+            nemo_file_set_monitor_number (file, item->monitor);
         } else {
-            str[0] = 0;
+            nemo_file_set_monitor_number (file, -1);
         }
 
-        nemo_file_set_metadata (file, NEMO_METADATA_KEY_MONITOR, NULL, str);
+        nemo_file_set_time_metadata (file, NEMO_METADATA_KEY_ICON_POSITION_TIMESTAMP, UNDEFINED_TIME);
 
 		nemo_file_unref (file);
 	}
@@ -1737,6 +1733,17 @@ nemo_directory_is_desktop_directory (NemoDirectory   *directory)
 	return nemo_is_desktop_directory (directory->details->location);
 }
 
+void
+nemo_directory_set_show_thumbnails (NemoDirectory         *directory,
+                                    gboolean show_thumbnails)
+{
+  NemoFile *file;
+  
+  file = nemo_file_get(directory->details->location);
+  nemo_file_set_boolean_metadata (file, NEMO_METADATA_KEY_SHOW_THUMBNAILS, FALSE, show_thumbnails);
+  nemo_directory_force_reload (directory);
+}
+
 #if !defined (NEMO_OMIT_SELF_CHECK)
 
 #include <eel/eel-debug.h>
