@@ -30,7 +30,6 @@ static char *used_metadata_names[] = {
   (char *)NEMO_METADATA_KEY_LOCATION_BACKGROUND_IMAGE,
   (char *)NEMO_METADATA_KEY_ICON_VIEW_ZOOM_LEVEL,
   (char *)NEMO_METADATA_KEY_ICON_VIEW_AUTO_LAYOUT,
-  (char *)NEMO_METADATA_KEY_ICON_VIEW_TIGHTER_LAYOUT,
   (char *)NEMO_METADATA_KEY_ICON_VIEW_SORT_BY,
   (char *)NEMO_METADATA_KEY_ICON_VIEW_SORT_REVERSED,
   (char *)NEMO_METADATA_KEY_ICON_VIEW_KEEP_ALIGNED,
@@ -59,6 +58,9 @@ static char *used_metadata_names[] = {
   (char *)NEMO_METADATA_KEY_EMBLEMS,
   (char *)NEMO_METADATA_KEY_MONITOR,
   (char *)NEMO_METADATA_KEY_DESKTOP_GRID_HORIZONTAL,
+  (char *)NEMO_METADATA_KEY_SHOW_THUMBNAILS,
+  (char *)NEMO_METADATA_KEY_DESKTOP_GRID_ADJUST,
+  (char *)NEMO_METADATA_KEY_PINNED,
   NULL
 };
 
