@@ -28,6 +28,7 @@
 #include "nemo-file.h"
 #include "nemo-file-private.h"
 #include "nemo-file-utilities.h"
+#include "nemo-global-preferences.h"
 #include "nemo-search-engine.h"
 #include <eel/eel-glib-extensions.h>
 #include <gtk/gtk.h>
@@ -139,6 +140,10 @@ start_or_stop_search_engine (NemoSearchDirectory *search, gboolean adding)
 		search->details->search_running = TRUE;
 		search->details->search_finished = FALSE;
 		ensure_search_engine (search);
+
+        nemo_query_set_show_hidden (search->details->query,
+                                    g_settings_get_boolean (nemo_preferences, NEMO_PREFERENCES_SHOW_HIDDEN_FILES));
+
 		nemo_search_engine_set_query (search->details->engine, search->details->query);
 
 		reset_file_list (search);
@@ -604,6 +609,10 @@ search_force_reload (NemoDirectory *directory)
 	
 	if (search->details->search_running) {
 		nemo_search_engine_stop (search->details->engine);
+
+        nemo_query_set_show_hidden (search->details->query,
+                                    g_settings_get_boolean (nemo_preferences, NEMO_PREFERENCES_SHOW_HIDDEN_FILES));
+
 		nemo_search_engine_set_query (search->details->engine, search->details->query);
 		nemo_search_engine_start (search->details->engine);
 	}
@@ -620,6 +629,16 @@ search_are_all_files_seen (NemoDirectory *directory)
 		search->details->search_finished);
 }
 
+static gboolean
+search_is_not_empty (NemoDirectory *directory)
+{
+	NemoSearchDirectory *search;
+
+	search = NEMO_SEARCH_DIRECTORY (directory);
+
+	return search->details->files != NULL;
+}
+
 static gboolean
 search_contains_file (NemoDirectory *directory,
 		      NemoFile *file)
@@ -697,7 +716,7 @@ search_dispose (GObject *object)
 		g_object_unref (search->details->engine);
 		search->details->engine = NULL;
 	}
-	
+
 	G_OBJECT_CLASS (nemo_search_directory_parent_class)->dispose (object);
 }
 
@@ -732,6 +751,7 @@ nemo_search_directory_class_init (NemoSearchDirectoryClass *class)
 	directory_class = NEMO_DIRECTORY_CLASS (class);
 
  	directory_class->are_all_files_seen = search_are_all_files_seen;
+	directory_class->is_not_empty = search_is_not_empty;
 	directory_class->contains_file = search_contains_file;
 	directory_class->force_reload = search_force_reload;
 	directory_class->call_when_ready = search_call_when_ready;
