@@ -182,9 +182,6 @@ static void                 nemo_icon_view_set_zoom_level               (NemoIco
 									     gboolean              always_emit);
 static void                 nemo_icon_view_update_click_mode            (NemoIconView           *icon_view);
 static void                 nemo_icon_view_update_click_to_rename_mode  (NemoIconView           *icon_view);
-static void                 nemo_icon_view_set_directory_tighter_layout (NemoIconView           *icon_view,
-                                        NemoFile         *file,
-                                        gboolean              tighter_layout);
 static gboolean             nemo_icon_view_is_desktop      (NemoIconView           *icon_view);
 static void                 nemo_icon_view_reveal_selection       (NemoView               *view);
 static const SortCriterion *get_sort_criterion_by_sort_type           (NemoFileSortType  sort_type);
@@ -349,27 +346,6 @@ action_clean_up_callback (GtkAction *action, gpointer callback_data)
 	nemo_icon_view_clean_up (NEMO_ICON_VIEW (callback_data));
 }
 
-static void
-set_tighter_layout (NemoIconView *icon_view, gboolean new_value)
-{
-   nemo_icon_view_set_directory_tighter_layout (icon_view,
-                          nemo_view_get_directory_as_file
-                          (NEMO_VIEW (icon_view)),
-                          new_value);
-   nemo_icon_container_set_tighter_layout (get_icon_container (icon_view),
-                           new_value);
-}
-
-static void
-action_tighter_layout_callback (GtkAction *action,
-               gpointer user_data)
-{
-   g_assert (NEMO_IS_ICON_VIEW (user_data));
-
-   set_tighter_layout (NEMO_ICON_VIEW (user_data),
-               gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)));
-}
-
 static gboolean
 nemo_icon_view_using_auto_layout (NemoIconView *icon_view)
 {
@@ -377,13 +353,6 @@ nemo_icon_view_using_auto_layout (NemoIconView *icon_view)
 		(get_icon_container (icon_view));
 }
 
-static gboolean
-nemo_icon_view_using_tighter_layout (NemoIconView *icon_view)
-{
-   return nemo_icon_container_is_tighter_layout
-       (get_icon_container (icon_view));
-}
-
 static void
 action_sort_radio_callback (GtkAction *action,
 			    GtkRadioAction *current,
@@ -503,13 +472,14 @@ nemo_icon_view_add_file (NemoView *view, NemoFile *file, NemoDirectory *director
 	g_assert (directory == nemo_view_get_model (view));
 
 	icon_view = NEMO_ICON_VIEW (view);
-	icon_container = get_icon_container (icon_view);
 
     if (icon_view->details->is_desktop &&
         !should_show_file_on_screen (view, file)) {
         return;
     }
 
+    icon_container = get_icon_container (icon_view);
+
     if (nemo_file_has_thumbnail_access_problem (file)) {
         nemo_application_set_cache_flag (nemo_application_get_singleton ());
         nemo_window_slot_check_bad_cache_bar (nemo_view_get_nemo_window_slot (view));
@@ -585,12 +555,6 @@ nemo_icon_view_supports_labels_beside_icons (NemoIconView *view)
 	return view->details->supports_labels_beside_icons;
 }
 
-static gboolean
-nemo_icon_view_supports_tighter_layout (NemoIconView *view)
-{
-   return !nemo_icon_view_is_compact (view);
-}
-
 static void
 update_layout_menus (NemoIconView *view)
 {
@@ -613,13 +577,6 @@ update_layout_menus (NemoIconView *view)
 						      action_name);
 		gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), TRUE);
 
-		action = gtk_action_group_get_action (view->details->icon_action_group,
-                                              NEMO_ACTION_TIGHTER_LAYOUT);
-        gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action),
-                          nemo_icon_view_using_tighter_layout (view));
-        gtk_action_set_sensitive (action, nemo_icon_view_supports_tighter_layout (view));
-        gtk_action_set_visible (action, nemo_icon_view_supports_tighter_layout (view));
-
         action = gtk_action_group_get_action (view->details->icon_action_group,
                                               NEMO_ACTION_REVERSED_ORDER);
 		gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action),
@@ -867,54 +824,38 @@ nemo_icon_view_get_directory_horizontal_layout (NemoIconView *icon_view,
                                            FALSE);
 }
 
-/* maintainence of tighter layout boolean */
-
-static gboolean
-get_default_directory_tighter_layout (void)
+void
+nemo_icon_view_set_directory_grid_adjusts (NemoIconView *icon_view,
+                                           NemoFile     *file,
+                                           gint          horizontal,
+                                           gint          vertical)
 {
-    return g_settings_get_boolean (nemo_icon_view_preferences,
-                      NEMO_PREFERENCES_ICON_VIEW_DEFAULT_USE_TIGHTER_LAYOUT);
+    sync_directory_monitor_number (icon_view, file);
+
+    nemo_file_set_desktop_grid_adjusts (file,
+                                        NEMO_METADATA_KEY_DESKTOP_GRID_ADJUST,
+                                        horizontal, vertical);
 }
 
-static gboolean
-nemo_icon_view_get_directory_tighter_layout (NemoIconView *icon_view,
-                        NemoFile *file)
+void
+nemo_icon_view_get_directory_grid_adjusts (NemoIconView *icon_view,
+                                           NemoFile     *file,
+                                           gint         *horizontal,
+                                           gint         *vertical)
 {
-    if (!nemo_icon_view_supports_tighter_layout (icon_view)) {
-        return FALSE;
-    }
-
-    if (nemo_global_preferences_get_ignore_view_metadata ()) {
-        gint t = nemo_window_get_ignore_meta_tighter_layout (nemo_view_get_nemo_window (NEMO_VIEW (icon_view)));
-        return t > TIGHTER_NULL ? t == TIGHTER_YES : get_default_directory_tighter_layout ();
-    }
+    gint h, v;
 
     sync_directory_monitor_number (icon_view, file);
 
-    return nemo_file_get_boolean_metadata (file,
-                                           NEMO_METADATA_KEY_ICON_VIEW_TIGHTER_LAYOUT,
-                                           get_default_directory_tighter_layout ());
-}
+    nemo_file_get_desktop_grid_adjusts (file,
+                                        NEMO_METADATA_KEY_DESKTOP_GRID_ADJUST,
+                                        &h, &v);
 
-static void
-nemo_icon_view_set_directory_tighter_layout (NemoIconView *icon_view,
-                        NemoFile *file,
-                        gboolean tighter_layout)
-{
-    if (!nemo_icon_view_supports_tighter_layout (icon_view)) {
-        return;
-    }
+    if (horizontal)
+        *horizontal = h;
 
-    if (nemo_global_preferences_get_ignore_view_metadata ()) {
-        gint t = tighter_layout ? TIGHTER_YES : TIGHTER_NO;
-        nemo_window_set_ignore_meta_tighter_layout (nemo_view_get_nemo_window (NEMO_VIEW (icon_view)), t);
-    } else {
-        sync_directory_monitor_number (icon_view, file);
-
-        nemo_file_set_boolean_metadata (file, NEMO_METADATA_KEY_ICON_VIEW_TIGHTER_LAYOUT,
-                                        get_default_directory_tighter_layout (),
-                                        tighter_layout);
-    }
+    if (vertical)
+        *vertical = v;
 }
 
 gboolean
@@ -986,6 +927,10 @@ get_default_zoom_level (NemoIconView *icon_view)
 	default_compact_zoom_level = g_settings_get_enum (nemo_compact_view_preferences,
 							  NEMO_PREFERENCES_COMPACT_VIEW_DEFAULT_ZOOM_LEVEL);
 
+    if (NEMO_ICON_VIEW_GET_CLASS (icon_view)->use_grid_container) {
+        return NEMO_ZOOM_LEVEL_STANDARD;
+    }
+
 	return CLAMP (DEFAULT_ZOOM_LEVEL(icon_view), NEMO_ZOOM_LEVEL_SMALLEST, NEMO_ZOOM_LEVEL_LARGEST);
 }
 
@@ -1030,6 +975,7 @@ nemo_icon_view_begin_loading (NemoView *view)
 	GtkWidget *icon_container;
 	NemoFile *file;
 	int level;
+    int h_adjust, v_adjust;
 	char *sort_name, *uri;
 
 	g_return_if_fail (NEMO_IS_ICON_VIEW (view));
@@ -1048,8 +994,11 @@ nemo_icon_view_begin_loading (NemoView *view)
 
 	/* Set up the zoom level from the metadata. */
 	if (nemo_view_supports_zooming (NEMO_VIEW (icon_view))) {
-        if (nemo_global_preferences_get_ignore_view_metadata () &&
-            nemo_window_get_ignore_meta_zoom_level (nemo_view_get_nemo_window (NEMO_VIEW (icon_view))) > -1) {
+        if (nemo_global_preferences_get_ignore_view_metadata () && !NEMO_ICON_VIEW_GET_CLASS (view)->use_grid_container) {
+            if (nemo_window_get_ignore_meta_zoom_level (nemo_view_get_nemo_window (view)) == -1) {
+                nemo_window_set_ignore_meta_zoom_level (nemo_view_get_nemo_window (view), get_default_zoom_level (icon_view));
+            }
+
             level = nemo_window_get_ignore_meta_zoom_level (nemo_view_get_nemo_window (NEMO_VIEW (icon_view)));
         } else {
             sync_directory_monitor_number (icon_view, file);
@@ -1085,8 +1034,12 @@ nemo_icon_view_begin_loading (NemoView *view)
 	nemo_icon_container_set_keep_aligned (get_icon_container (icon_view),
                     nemo_icon_view_get_directory_keep_aligned (icon_view, file));
 
-    nemo_icon_container_set_tighter_layout (get_icon_container (icon_view),
-                    nemo_icon_view_get_directory_tighter_layout (icon_view, file));
+    nemo_icon_view_get_directory_grid_adjusts (NEMO_ICON_VIEW (view),
+                                               file,
+                                               &h_adjust,
+                                               &v_adjust);
+
+    nemo_icon_container_set_grid_adjusts (get_icon_container (icon_view), h_adjust, v_adjust);
 
 	set_labels_beside_icons (icon_view);
 	set_columns_same_width (icon_view);
@@ -1133,6 +1086,8 @@ nemo_icon_view_end_loading (NemoView *view,
 	icon_container = GTK_WIDGET (get_icon_container (icon_view));
 	nemo_icon_container_end_loading (NEMO_ICON_CONTAINER (icon_container), all_files_seen);
 
+    nemo_icon_container_update_selection (NEMO_ICON_CONTAINER (icon_container));
+
 	monitor = nemo_clipboard_monitor_get ();
 	info = nemo_clipboard_monitor_get_clipboard_info (monitor);
 
@@ -1166,7 +1121,7 @@ nemo_icon_view_set_zoom_level (NemoIconView *view,
 		return;
 	}
 
-    if (nemo_global_preferences_get_ignore_view_metadata ()) {
+    if (nemo_global_preferences_get_ignore_view_metadata () && !NEMO_ICON_VIEW_GET_CLASS (view)->use_grid_container) {
         nemo_window_set_ignore_meta_zoom_level (nemo_view_get_nemo_window (NEMO_VIEW (view)), new_level);
     } else {
         sync_directory_monitor_number (view, nemo_view_get_directory_as_file (NEMO_VIEW (view)));
@@ -1280,6 +1235,26 @@ nemo_icon_view_get_selection (NemoView *view)
 	return list;
 }
 
+static GList *
+nemo_icon_view_peek_selection (NemoView *view)
+{
+    GList *list;
+
+    g_return_val_if_fail (NEMO_IS_ICON_VIEW (view), NULL);
+
+    list = nemo_icon_container_peek_selection (get_icon_container (NEMO_ICON_VIEW (view)));
+    nemo_file_list_ref (list);
+    return list;
+}
+
+static gint
+nemo_icon_view_get_selection_count (NemoView *view)
+{
+    g_return_val_if_fail (NEMO_IS_ICON_VIEW (view), 0);
+
+    return nemo_icon_container_get_selection_count (get_icon_container (NEMO_ICON_VIEW (view)));
+}
+
 static void
 count_item (NemoIconData *icon_data,
 	    gpointer callback_data)
@@ -1394,11 +1369,6 @@ layout_changed_callback (NemoIconContainer *container,
 			(icon_view,
 			 file,
 			 nemo_icon_view_using_auto_layout (icon_view));
-
-        nemo_icon_view_set_directory_tighter_layout
-           (icon_view,
-            file,
-            nemo_icon_view_using_tighter_layout (icon_view));
 	}
 
 	update_layout_menus (icon_view);
@@ -1437,11 +1407,6 @@ static const GtkActionEntry icon_view_entries[] = {
 
 static const GtkToggleActionEntry icon_view_toggle_entries[] = {
 
-  /* name, stock id */      { "Tighter Layout", NULL,
-  /* label, accelerator */    N_("Compact _Layout"), NULL,
-  /* tooltip */               N_("Toggle using a tighter layout scheme"),
-                              G_CALLBACK (action_tighter_layout_callback),
-                              0 },
   /* name, stock id */      { "Reversed Order", NULL,
   /* label, accelerator */    N_("Re_versed Order"), NULL,
   /* tooltip */               N_("Display icons in the opposite order"),
@@ -1582,8 +1547,6 @@ nemo_icon_view_reset_to_defaults (NemoView *view)
 	clear_sort_criterion (icon_view);
 	nemo_icon_container_set_keep_aligned
 		(icon_container, get_default_directory_keep_aligned ());
-    nemo_icon_container_set_tighter_layout
-       (icon_container, get_default_directory_tighter_layout ());
 
 	nemo_icon_container_sort (icon_container);
 
@@ -1593,7 +1556,6 @@ nemo_icon_view_reset_to_defaults (NemoView *view)
 
     if (nemo_global_preferences_get_ignore_view_metadata ()) {
         NemoWindow *window = nemo_view_get_nemo_window (view);
-        nemo_window_set_ignore_meta_tighter_layout (window, TIGHTER_NULL);
         nemo_window_set_ignore_meta_zoom_level (window, NEMO_ZOOM_LEVEL_NULL);
     }
 }
@@ -1906,6 +1868,8 @@ selection_changed_callback (NemoIconContainer *container,
 	g_assert (NEMO_IS_ICON_VIEW (icon_view));
 	g_assert (container == get_icon_container (icon_view));
 
+    nemo_icon_container_update_selection (container);
+
 	nemo_view_notify_selection_changed (NEMO_VIEW (icon_view));
 }
 
@@ -2129,27 +2093,6 @@ default_sort_in_reverse_order_changed_callback (gpointer callback_data)
 	nemo_icon_container_request_update_all (icon_container);
 }
 
-static void
-default_use_tighter_layout_changed_callback (gpointer callback_data)
-{
-   NemoIconView *icon_view;
-   NemoFile *file;
-   NemoIconContainer *icon_container;
-
-   g_return_if_fail (NEMO_IS_ICON_VIEW (callback_data));
-
-   icon_view = NEMO_ICON_VIEW (callback_data);
-
-   file = nemo_view_get_directory_as_file (NEMO_VIEW (icon_view));
-   icon_container = get_icon_container (icon_view);
-   g_return_if_fail (NEMO_IS_ICON_CONTAINER (icon_container));
-
-   nemo_icon_container_set_tighter_layout (icon_container,
-                           nemo_icon_view_get_directory_tighter_layout (icon_view, file));
-
-   nemo_icon_container_request_update_all (icon_container);
-}
-
 static void
 default_zoom_level_changed_callback (gpointer callback_data)
 {
@@ -2568,7 +2511,7 @@ set_compact_view (NemoIconView *icon_view,
                                                                                                      NEMO_ICON_LAYOUT_T_B_R_L :
                                                                                                      NEMO_ICON_LAYOUT_T_B_L_R);
         nemo_icon_container_set_forced_icon_size (get_icon_container (icon_view),
-                                                  NEMO_ICON_SIZE_SMALLEST);
+                                                  NEMO_COMPACT_FORCED_ICON_SIZE);
     } else {
         nemo_icon_container_set_layout_mode (get_icon_container (icon_view),
                                              gtk_widget_get_direction (GTK_WIDGET(icon_view)) == GTK_TEXT_DIR_RTL ?
@@ -2630,10 +2573,6 @@ nemo_icon_view_finalize (GObject *object)
 					      image_display_policy_changed_callback,
 					      icon_view);
 
-    g_signal_handlers_disconnect_by_func (nemo_icon_view_preferences,
-                          default_use_tighter_layout_changed_callback,
-                          icon_view);
-
 	g_signal_handlers_disconnect_by_func (nemo_icon_view_preferences,
 					      default_zoom_level_changed_callback,
 					      icon_view);
@@ -2655,10 +2594,13 @@ nemo_icon_view_finalize (GObject *object)
 }
 
 static void
-nemo_icon_view_constructed (NemoIconView *icon_view)
+nemo_icon_view_constructed (GObject *object)
 {
+    NemoIconView *icon_view;
     NemoIconContainer *icon_container;
 
+    icon_view = NEMO_ICON_VIEW (object);
+
     G_OBJECT_CLASS (nemo_icon_view_parent_class)->constructed (G_OBJECT (icon_view));
 
     g_return_if_fail (gtk_bin_get_child (GTK_BIN (icon_view)) == NULL);
@@ -2685,11 +2627,6 @@ nemo_icon_view_constructed (NemoIconView *icon_view)
                   "changed::" NEMO_PREFERENCES_SHOW_IMAGE_FILE_THUMBNAILS,
                   G_CALLBACK (image_display_policy_changed_callback),
                   icon_view);
-    g_signal_connect_swapped (nemo_icon_view_preferences,
-                 "changed::" NEMO_PREFERENCES_ICON_VIEW_DEFAULT_USE_TIGHTER_LAYOUT,
-                 G_CALLBACK (default_use_tighter_layout_changed_callback),
-                 icon_view);
-
     g_signal_connect_swapped (nemo_icon_view_preferences,
                   "changed::" NEMO_PREFERENCES_ICON_VIEW_DEFAULT_ZOOM_LEVEL,
                   G_CALLBACK (default_zoom_level_changed_callback),
@@ -2758,7 +2695,9 @@ nemo_icon_view_class_init (NemoIconViewClass *klass)
 	nemo_view_class->end_loading = nemo_icon_view_end_loading;
 	nemo_view_class->file_changed = nemo_icon_view_file_changed;
 	nemo_view_class->get_selected_icon_locations = nemo_icon_view_get_selected_icon_locations;
-	nemo_view_class->get_selection = nemo_icon_view_get_selection;
+    nemo_view_class->get_selection = nemo_icon_view_get_selection;
+    nemo_view_class->peek_selection = nemo_icon_view_peek_selection;
+	nemo_view_class->get_selection_count = nemo_icon_view_get_selection_count;
 	nemo_view_class->get_selection_for_file_transfer = nemo_icon_view_get_selection;
 	nemo_view_class->get_item_count = nemo_icon_view_get_item_count;
 	nemo_view_class->is_empty = nemo_icon_view_is_empty;
