@@ -41,8 +41,6 @@
 #define AFTER_ALL_TABS -1
 #define NOT_IN_APP_WINDOWS -2
 
-static void nemo_notebook_init		 (NemoNotebook *notebook);
-static void nemo_notebook_class_init	 (NemoNotebookClass *klass);
 static int  nemo_notebook_insert_page	 (GtkNotebook *notebook,
 					  GtkWidget *child,
 					  GtkWidget *tab_label,
@@ -51,14 +49,6 @@ static int  nemo_notebook_insert_page	 (GtkNotebook *notebook,
 static void nemo_notebook_remove	 (GtkContainer *container,
 					  GtkWidget *tab_widget);
 
-#if 0
-static const GtkTargetEntry url_drag_types[] =
-{
-	{ NEMO_ICON_DND_GNOME_ICON_LIST_TYPE, 0, NEMO_ICON_DND_GNOME_ICON_LIST },
-	{ NEMO_ICON_DND_URI_LIST_TYPE, 0, NEMO_ICON_DND_URI_LIST },
-};
-#endif
-
 enum
 {
 	TAB_CLOSE_REQUEST,
@@ -259,19 +249,6 @@ nemo_notebook_init (NemoNotebook *notebook)
 			  G_CALLBACK(control_key_checker_cb), GINT_TO_POINTER (TRUE));
 	g_signal_connect (notebook, "key-release-event",
 			  G_CALLBACK(control_key_checker_cb), GINT_TO_POINTER (FALSE));
-
-	/* Set up drag-and-drop target */
-	/* TODO this would be used for opening a new tab.
-	 * It will only work properly as soon as GtkNotebook
-	 * supports to find out whether a particular point
-	 * is on a tab button or not.
-	 */
-#if 0
-	gtk_drag_dest_set (GTK_WIDGET (notebook), 0,
-			   url_drag_types, G_N_ELEMENTS (url_drag_types),
-			   GDK_ACTION_LINK);
-	gtk_drag_dest_set_track_motion (GTK_WIDGET (notebook), TRUE);
-#endif
 }
 
 void
@@ -388,7 +365,7 @@ build_tab_label (NemoNotebook *nb, NemoWindowSlot *slot)
 
 	gtk_widget_set_name (close_button, "nemo-tab-close-button");
 
-	image = gtk_image_new_from_icon_name ("window-close", GTK_ICON_SIZE_MENU);
+	image = gtk_image_new_from_icon_name ("window-close-symbolic", GTK_ICON_SIZE_MENU);
 	gtk_widget_set_tooltip_text (close_button, _("Close tab"));
 	g_signal_connect_object (close_button, "clicked",
 				 G_CALLBACK (close_button_clicked_cb), slot, 0);
