@@ -93,6 +93,7 @@ struct TreeNode {
 	guint done_loading : 1;
 	guint force_has_dummy : 1;
 	guint inserted : 1;
+    guint pinned : 1;
 };
 
 struct FMTreeModelDetails {
@@ -270,30 +271,22 @@ get_menu_icon_for_file (TreeNode *node,
                         NemoFile *file,
                         NemoFileIconFlags flags)
 {
+    NemoFile *parent_file;
 	GIcon *gicon, *emblem_icon, *emblemed_icon;
 	GEmblem *emblem;
 	int size;
 	GList *emblem_icons, *l;
-	char *emblems_to_ignore[3];
-	int i;
 
 	size = nemo_get_icon_size_for_stock_size (GTK_ICON_SIZE_MENU);
 	gicon = G_ICON (nemo_file_get_icon_pixbuf (file, size, TRUE, node->icon_scale, flags));
 
-	i = 0;
-	emblems_to_ignore[i++] = NEMO_FILE_EMBLEM_NAME_TRASH;
+    parent_file = NULL;
 
 	if (node->parent && node->parent->file) {
-		if (!nemo_file_can_write (node->parent->file)) {
-			emblems_to_ignore[i++] = NEMO_FILE_EMBLEM_NAME_CANT_WRITE;
-		}
+        parent_file = node->parent->file;
 	}
 
-	emblems_to_ignore[i++] = NULL;
-
-	emblem = NULL;
-	emblem_icons = nemo_file_get_emblem_icons (node->file,
-						       emblems_to_ignore);
+	emblem_icons = nemo_file_get_emblem_icons (node->file, parent_file);
 
 	/* pick only the first emblem we can render for the tree view */
 	for (l = emblem_icons; l != NULL; l = l->next) {
@@ -377,6 +370,22 @@ tree_node_update_display_name (TreeNode *node)
 	return TRUE;
 }
 
+static gboolean
+tree_node_update_pinning (TreeNode *node)
+{
+    gboolean file_value;
+
+    file_value = nemo_file_get_pinning (node->file);
+
+    if (file_value != node->pinned) {
+        node->pinned = file_value;
+
+        return TRUE;
+    }
+
+    return FALSE;
+}
+
 static GIcon *
 tree_node_get_closed_icon (TreeNode *node)
 {
@@ -818,6 +827,7 @@ update_node_without_reporting (FMTreeModel *model, TreeNode *node)
 	changed |= tree_node_update_display_name (node);
 	changed |= tree_node_update_closed_icon (node);
     changed |= tree_node_update_open_icon (node);
+    changed |= tree_node_update_pinning (node);
 
 	return changed;
 }
@@ -1111,6 +1121,8 @@ fm_tree_model_get_column_type (GtkTreeModel *model, int index)
 		return G_TYPE_ICON;
 	case FM_TREE_MODEL_FONT_STYLE_COLUMN:
 		return PANGO_TYPE_STYLE;
+    case FM_TREE_MODEL_TEXT_WEIGHT_COLUMN:
+        return G_TYPE_INT;
 	default:
 		g_assert_not_reached ();
 	}
@@ -1258,6 +1270,16 @@ fm_tree_model_get_value (GtkTreeModel *model, GtkTreeIter *iter, int column, GVa
 			g_value_set_enum (value, PANGO_STYLE_NORMAL);
 		}
 		break;
+    case FM_TREE_MODEL_TEXT_WEIGHT_COLUMN:
+        g_value_init (value, G_TYPE_INT);
+
+        if (node != NULL) {
+            g_value_set_int (value, node->pinned ? PINNED_TEXT_WEIGHT : UNPINNED_TEXT_WEIGHT);
+        } else {
+            g_value_set_int (value, UNPINNED_TEXT_WEIGHT);
+        }
+
+        break;
 	default:
 		g_assert_not_reached ();
 	}
