@@ -109,6 +109,8 @@ struct NemoWindowSlot {
 	NemoWindowGoToCallback open_callback;
 	gpointer open_callback_user_data;
 
+	gboolean needs_reload;
+
 	GCancellable *find_mount_cancellable;
 
 	gboolean visible;
@@ -127,13 +129,13 @@ void    nemo_window_slot_update_title		   (NemoWindowSlot *slot);
 void    nemo_window_slot_update_icon		   (NemoWindowSlot *slot);
 void    nemo_window_slot_set_query_editor_visible	   (NemoWindowSlot *slot,
 							    gboolean            visible);
-gboolean nemo_window_slot_handle_event       	   (NemoWindowSlot *slot,
-							    GdkEventKey        *event);
 
 GFile * nemo_window_slot_get_location		   (NemoWindowSlot *slot);
 char *  nemo_window_slot_get_location_uri		   (NemoWindowSlot *slot);
 
-void    nemo_window_slot_reload			   (NemoWindowSlot *slot);
+void nemo_window_slot_queue_reload (NemoWindowSlot *slot,
+                                    gboolean        clear_thumbs);
+void nemo_window_slot_force_reload (NemoWindowSlot *slot);
 
 /* convenience wrapper without selection and callback/user_data */
 #define nemo_window_slot_open_location(slot, location, flags)\
@@ -186,4 +188,6 @@ void nemo_window_slot_clear_back_list    (NemoWindowSlot *slot);
 
 void nemo_window_slot_check_bad_cache_bar (NemoWindowSlot *slot);
 
+void nemo_window_slot_set_show_thumbnails (NemoWindowSlot *slot,
+                                           gboolean show_thumbnails);
 #endif /* NEMO_WINDOW_SLOT_H */
